// AUTO-GENERATED BY UpdateGenVersion.bat ba923cd412664ec5cd0b630548f74a7fc59069c8
const char GenVersion_Version[] = "0.4.4-dev";
const char GenVersion_Commit[] = "ba923cd412664ec5cd0b630548f74a7fc59069c8";
